package com.soss.system.service.impl;

import com.soss.common.exception.ServiceException;
import com.soss.common.utils.StringUtils;
import com.soss.system.domain.Shop;
import com.soss.system.domain.SysBanner;
import com.soss.system.domain.po.BannerPo;
import com.soss.system.mapper.SysBannerMapper;
import com.soss.system.service.ISysBannerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 轮播图Service业务层处理
 *
 * @author zxq
 * @date 2022-04-28
 */
@Service
@Slf4j
public class SysBannerServiceImpl implements ISysBannerService {
    @Autowired
    private SysBannerMapper sysBannerMapper;
    @Autowired
    private ShopServiceImpl shopService;

    @Override
    public List<SysBanner> selectBanner(SysBanner banner) {
        return sysBannerMapper.selectBanner(banner);
    }

    @Override
    public Long createBanner(BannerPo bannerPo) {
        SysBanner banner = copyBanner(bannerPo);
        if (StringUtils.isEmpty(banner.getImage())) {
            throw new ServiceException("图片地址不能为空");
        }
//        if (StringUtils.isEmpty(banner.getPage())) {
//            throw new ServiceException("链接地址不能为空");
//        }
        if (banner.getType() == null || banner.getType() < 1 || banner.getType() > 3) {
            throw new ServiceException("类型不正确，1-小程序，2-KDS，3-点单屏");
        }
        if (banner.getTarget() == null || banner.getTarget() < 1 || banner.getTarget() > 2) {
            throw new ServiceException("轮播位置不正确，1-小程序首页，2-小程序我的");
        }
        if (banner.getSort() == null) {
            throw new ServiceException("轮播顺序不能为空");
        }
        if (StringUtils.isEmpty(banner.getProvince()) && StringUtils.isEmpty(banner.getCity()) && StringUtils.isEmpty(banner.getArea()) && StringUtils.isEmpty(banner.getShopIds())) {
            throw new ServiceException("适用范围不能为空");
        }

        banner.setCreatedAt(new Date());
        banner.setUpdatedAt(new Date());
        banner.setState(0);
        long i = sysBannerMapper.createBanner(banner);
        return banner.getId();
    }

    @Override
    public Boolean updateBanner(BannerPo bannerPo) {
        SysBanner banner = copyBanner(bannerPo);
        if (banner.getId() == null || banner.getId() <= 0) {
            throw new ServiceException("id不能为空");
        }
        if (StringUtils.isEmpty(banner.getImage())) {
            throw new ServiceException("图片地址不能为空");
        }
//        if (StringUtils.isEmpty(banner.getPage())) {
//            throw new ServiceException("链接地址不能为空");
//        }
        if (banner.getType() == null || banner.getType() < 1 || banner.getType() > 3) {
            throw new ServiceException("类型不正确，1-小程序，2-KDS，3-点单屏");
        }
        if (banner.getTarget() == null || banner.getTarget() < 1 || banner.getTarget() > 2) {
            throw new ServiceException("轮播位置不正确，1-小程序首页，2-小程序我的");
        }
        if (banner.getSort() == null) {
            throw new ServiceException("轮播顺序不能为空");
        }
        if (StringUtils.isEmpty(banner.getProvince()) && StringUtils.isEmpty(banner.getCity()) && StringUtils.isEmpty(banner.getArea()) && StringUtils.isEmpty(banner.getShopIds())) {
            throw new ServiceException("适用范围不能为空");
        }

        banner.setUpdatedAt(new Date());

        long i = sysBannerMapper.updateBanner(banner);
        return i > 0 ? true : false;
    }

    private SysBanner copyBanner(BannerPo bannerPo) {
        SysBanner banner = new SysBanner();
        banner.setId(bannerPo.getId());
        banner.setImage(bannerPo.getImage());
        banner.setPage(bannerPo.getPage());
        banner.setType(bannerPo.getType());
        banner.setTarget(bannerPo.getTarget());
        banner.setSort(bannerPo.getSort());
        if (bannerPo.getProvince() != null && bannerPo.getProvince().length > 0) {
            banner.setProvince(Arrays.asList(bannerPo.getProvince()).stream().collect(Collectors.joining(",")));
        }
        if (bannerPo.getCity() != null && bannerPo.getCity().length > 0) {
            banner.setCity(Arrays.asList(bannerPo.getCity()).stream().collect(Collectors.joining(",")));
        }
        if (bannerPo.getArea() != null && bannerPo.getArea().length > 0) {
            banner.setArea(Arrays.asList(bannerPo.getArea()).stream().collect(Collectors.joining(",")));
        }
        if (bannerPo.getShopIds() != null && bannerPo.getShopIds().length > 0) {
            banner.setShopIds(Arrays.asList(bannerPo.getShopIds()).stream().map(x -> String.valueOf(x)).collect(Collectors.joining(",")));
        }

        return banner;
    }

    @Override
    public Boolean updateState(Long bannerId, Integer state) {
        SysBanner banner = new SysBanner();
        banner.setId(bannerId);
        banner.setState(state);
        banner.setUpdatedAt(new Date());

        if (banner.getId() == null || banner.getId() <= 0) {
            throw new ServiceException("id不能为空");
        }
        if (banner.getState() == null || banner.getState() < 0 || banner.getState() > 1) {
            throw new ServiceException("状态不正确，0-上线，1-下线");
        }

        long i = sysBannerMapper.updateState(banner);
        return i > 0 ? true : false;
    }

    @Override
    public Boolean delete(Long bannerId) {
        long i = sysBannerMapper.delete(bannerId);
        return i > 0 ? true : false;
    }

    @Override
    public List<SysBanner> getBannersByShopId(Long shopId, Integer type, Integer target) {
        List<SysBanner> banners = sysBannerMapper.selectBanners(type, target);
        if (CollectionUtils.isEmpty(banners)) {
            log.info("banner list is empty");
            return null;
        }
        Shop shop = shopService.selectShopById(shopId);
        if (shop == null) {
            log.info("shop is null");
            return null;
        }
        List<SysBanner> list = new ArrayList<>();
        banners.forEach(banner -> {
            if (!StringUtils.isEmpty(banner.getProvince())) {
                Arrays.stream(banner.getProvince().split(",")).forEach(p -> {
                    if (p.equals(shop.getProvince())) {
                        list.add(banner);
                        return;
                    }
                });
            }
            if (!StringUtils.isEmpty(banner.getCity())) {
                Arrays.stream(banner.getCity().split(",")).forEach(p -> {
                    if (p.equals(shop.getCity())) {
                        list.add(banner);
                        return;
                    }
                });
            }
            if (!StringUtils.isEmpty(banner.getArea())) {
                Arrays.stream(banner.getArea().split(",")).forEach(p -> {
                    if (p.equals(shop.getZone())) {
                        list.add(banner);
                        return;
                    }
                });
            }
            if (!StringUtils.isEmpty(banner.getShopIds())) {
                Arrays.stream(banner.getShopIds().split(",")).forEach(p -> {
                    if (p.equals(String.valueOf(shop.getId()))) {
                        list.add(banner);
                        return;
                    }
                });
            }
        });
        return list;
    }
}
